//+------------------------------------------------------------------+
//|                                                    CanvasBar.mq5 |
//|                                  Copyright 2023, MetaQuotes Ltd. |
//|                                             https://www.mql5.com |
//+------------------------------------------------------------------+
#property copyright "Copyright 2023, MetaQuotes Ltd."
#property link      "https://www.mql5.com"
#property version   "1.00"
#include <Canvas\iCanvas_CB.mqh> //https://www.mql5.com/ru/code/22164

input bool erase=false; // clean out the main bars?
MqlRates Bar0;
uint clrBull=0x8000FF00;
uint clrBear=0x80FF0000;
uint clrLine=0xFFFF00FF;
//+------------------------------------------------------------------+
//| Expert initialization function                                   |
//+------------------------------------------------------------------+
int OnInit() {
   ChartSetInteger(0,CHART_FOREGROUND,false);
   if(erase) ChartSetInteger(0,CHART_SHOW,false);
   ShowBars();
   return(INIT_SUCCEEDED);
}
//+------------------------------------------------------------------+
//| Expert deinitialization function                                 |
//+------------------------------------------------------------------+
void OnDeinit(const int reason) {
   if(erase) ChartSetInteger(0,CHART_SHOW,true);
}
//+------------------------------------------------------------------+
//| Expert tick function                                             |
//+------------------------------------------------------------------+
void OnTick() {
   static int bars = 0;
   int cur_bars = Bars(_Symbol,PERIOD_CURRENT);
   if(cur_bars == bars) {
      Bar0.close=iClose(_Symbol,0,0);
      Bar0.open=iOpen(_Symbol,0,0);
      Bar0.high=iHigh(_Symbol,0,0);
      Bar0.low=iLow(_Symbol,0,0);
      ShowBars(false);
   } else  bars =  cur_bars;
}
//+------------------------------------------------------------------+
//| ChartEvent function                                              |
//+------------------------------------------------------------------+
void OnChartEvent(const int id,
                  const long &lparam,
                  const double &dparam,
                  const string &sparam) {
   if(id==CHARTEVENT_CHART_CHANGE) ShowBars(true);
}
//+------------------------------------------------------------------+
void ShowBars(bool change=true) {

   Canvas.Erase(W.Color);
   //ulong tt = GetMicrosecondCount();
   static MqlRates B[];
   uint clrBody=0;
   if(change) CopyRates(_Symbol,_Period,(int)W.Right_bar,W.BarsInWind,B);
   else B[ArraySize(B)-1]=Bar0;
   for(int i=0,x=-1; i<W.BarsInWind; i++,x+=W.dx_pix) {
      Canvas.LineVertical(x,(int)(Canvas.Y(B[i].high)-0.5),(int)(Canvas.Y(B[i].low)-0.5),clrLine);
      if(B[i].close>B[i].open) clrBody=clrBull;
      else if(B[i].close<B[i].open) clrBody=clrBear;
      else clrBody=clrLine;
      Canvas.FillRectangle(x-int(W.dx_pix*0.35),(int)(Canvas.Y(B[i].open)-0.5),x+int(W.dx_pix*0.35),(int)(Canvas.Y(B[i].close)-0.5),clrBody);
   }
   //tt = GetMicrosecondCount()-tt;
   Canvas.Update();

   //Print(DoubleToString(tt/1000.0,3)+ " milliseconds");
}
//+------------------------------------------------------------------+
//+------------------------------------------------------------------+
